const logger = require('../../lib/logger')
const globby = require('globby-extra')
const Promise = require('bluebird')
const NOS = require('../../src/nos')
const path = require('path')

exports.command = 'upload'
exports.desc = 'upload files to nos'
exports.builder = function (yargs) {
  // 检查 config
  return yargs.check(require('../../lib/checkConfig'))
}
exports.handler = function (argv) {
  const {
    host,
    port,
    bucket,
    accessId,
    secretKey,
    prefix,
    pattern,
    context,
    concurrency
  } = argv
  let paths
  // globby 后上传每一项
  globby(pattern, context).then(arr => {
    const nos = new NOS({
      host,
      port,
      bucket,
      accessId,
      secretKey
    })
    paths = arr
    logger.log('uploading %d', paths.length)
    return Promise.map(paths, (filePath, index) => {
      // key: prefix + path (without context)
      const contextPrefix = path.resolve(process.cwd(), context || '.')
      const filePathUri = filePath.replace(contextPrefix, '')
      const key = path.join(prefix, filePathUri)
      logger.info('uploading %d %s to %s', index, filePathUri, key)
      return nos.putFilePathAsync({
        key: key,
        filePath
      }).then(() => {
        logger.success('uploaded %d %s to %s', index, filePathUri, key)
      })
    }, {
      concurrency
    })
  }).then(() => {
    logger.success('upload %s done', paths.length)
  }, err => {
    logger.error(err.message, err)
  })
}
