const logger = require('../lib/logger')
const path = require('path')
const fs = require('fs')

const DefaultJSONConfigPath = '.nosrc'
const DefaultJSConfigPath = DefaultJSONConfigPath + '.js'

function checkConfig (argv, alias) {
  let {config: configPath} = argv
  let config = {}

  // 如果没有提供配置地址, 那么看看是否有默认的 js 配置文件
  if (!configPath) {
    try {
      if (fs.statSync(DefaultJSConfigPath)) {
        logger.warn('detect js config file %s', DefaultJSConfigPath)
        configPath = DefaultJSConfigPath
      }
    } catch (err) {
    }
  }
  // 如果没有提供配置地址, 那么看看是否有默认的 json 配置文件
  if (!configPath) {
    try {
      if (fs.statSync(DefaultJSONConfigPath)) {
        logger.warn('detect json config file %s', DefaultJSONConfigPath)
        configPath = DefaultJSONConfigPath
      }
    } catch (err) {
    }
  }

  // 如果依然没有配置地址, 报错
  if (!configPath) {
    throw new Error('please provide a config file')
  }

  try {
    if (configPath.lastIndexOf('.js') === configPath.length - 3) {
      // config = module.exports
      config = require(path.resolve(configPath))
    } else {
      // config = json
      config = JSON.parse(fs.readFileSync(path.resolve(configPath), 'utf-8'))
    }
    // 构造 prefix = prefix + versionPath.version || version
    const {context = '.', prefix = '', versionPath = ''} = config
    let {version = ''} = config
    if (versionPath) {
      version = require(path.resolve(context, versionPath)).version
    }
    config.prefix = path.join(prefix, version)
  } catch (e) {
    logger.error('can not parse JSON config file')
  }

  // 将 config 赋给 argv
  logger.log('parsed config', config)
  Object.assign(argv, config)
  return true
}

module.exports = checkConfig
