const NOS = require('./index')
const parseXml = require('xml2js').parseString
const fs = require('fs-extra-promise')
var crypto = require('crypto')

function NOSAsync (options) {
  this.setHost(options.host)
  this.setPort(options.port)
  this.setAccessId(options.accessId)
  this.setSecretKey(options.secretKey)
  this.bucket = options.bucket
}

const proto = NOSAsync.prototype = Object.create(NOS.prototype)

proto.cb = function (options, res) {
  const {resolve, reject, resType} = options
  // 处理 headers
  const headers = res.headers
  var nameHeader = 'x-nos-object-name'
  if (headers[nameHeader]) {
    headers[nameHeader] = decodeURIComponent(headers[nameHeader])
  }

  res.on('error', Error => {
    reject({Error, headers})
  })

  // 接收数据
  res.setEncoding('utf8')
  let body = ''
  res.on('data', chunk => {
    body += chunk
  })

  // 处理结果
  res.on('end', () => {
    if (resType === 'xml' && body) {
      parseXml(body, (Error, obj) => {
        if (Error) {
          reject({Error, headers})
        } else {
          obj = obj || {}
          obj.headers = headers
          if (obj.Error) {
            reject(obj)
          } else {
            resolve(obj)
          }
        }
      })
    } else if (resType === 'file') {
      const md5 = crypto.createHash('md5').update(body).digest('hex')
      const {filePath} = options
      if (filePath) {
        fs.writeFileAsync(filePath, body).then(() => {
          resolve({headers})
        }, Error => {
          reject({Error, headers})
        })
      } else {
        resolve({body, headers})
      }
    } else {
      resolve({headers})
    }
  })
}

proto.putObjectAsync = function (options) {
  return fs.readFileAsync(options.filePath).then(buffer => {
    options.bucket = this.bucket
    options.body = buffer
    return new Promise((resolve, reject) => {
      this.put_object(options, this.cb.bind(this, {
        resolve,
        reject,
        resType: 'xml'
      }))
    })
  })
}

proto.copyObjectAsync = function (options) {
  options.src_bucket = this.bucket
  options.dest_bucket = this.bucket
  return new Promise((resolve, reject) => {
    this.copy_object(options, this.cb.bind(this, {
      resolve,
      reject
    }))
  })
}

proto.moveObjectAsync = function (options) {
  options.src_bucket = this.bucket
  options.dest_bucket = this.bucket
  return new Promise((resolve, reject) => {
    this.move_object(options, this.cb.bind(this, {
      resolve,
      reject
    }))
  })
}

proto.headObjectAsync = function (options) {
  options.bucket = this.bucket
  return new Promise((resolve, reject) => {
    this.head_object(options, this.cb.bind(this, {
      resolve,
      reject
    }))
  })
}

proto.getObjectAsync = function (options) {
  options.bucket = this.bucket
  return new Promise((resolve, reject) => {
    this.get_object(options, this.cb.bind(this, {
      resolve,
      reject,
      resType: 'file',
      filePath: options.filePath
    }))
  })
}

proto.listObjectsAsync = function (options) {
  options = options || {}
  options.bucket = this.bucket
  return new Promise((resolve, reject) => {
    this.list_objects(options, this.cb.bind(this, {
      resolve,
      reject,
      resType: 'xml'
    }))
  })
}

proto.deleteObjectAsync = function (key) {
  var options = {
    bucket: this.bucket,
    key: key
  }
  return new Promise((resolve, reject) => {
    this.delete_object(options, this.cb.bind(this, {
      resolve,
      reject,
      resType: 'xml'
    }))
  })
}

proto.deleteObjectsAsync = function (options) {
  options.quiet = options.quiet ? 'True' : 'False'
  options.bucket = this.bucket
  options.keys = options.keys.map(key => {
    return {Key: key}
  })
  return new Promise((resolve, reject) => {
    this.delete_objects(options, this.cb.bind(this, {
      resolve,
      reject,
      resType: 'xml'
    }))
  })
}

module.exports = NOSAsync
