/**
 * Created by hzlichaolin on 2016/7/12.
 * this module define the operation that users can do
 */
var getObjectRequest = require('./lib/httpRequest/getObjectRequest')
var deleteObjectRequest = require('./lib/httpRequest/deleteObjectRequest')
var deleteMultipleObjectsRequest = require('./lib/httpRequest/deleteMultipleObjectsRequest')
var headObjectRequest = require('./lib/httpRequest/headObjectRequest')
var listObjectsRequest = require('./lib/httpRequest/listObjectsRequest')
var putObjectRequest = require('./lib/httpRequest/putObjectRequest')
var putObjectMoveRequest = require('./lib/httpRequest/putObjectMoveRequest')
var putObjectCopyRequest = require('./lib/httpRequest/putObjectCopyRequest')
var initiateMultipartUploadRequest = require('./lib/httpRequest/initiateMultipartUploadRequest')
var uploadPartRequest = require('./lib/httpRequest/uploadPartRequest')
var abortUploadPartRequest = require('./lib/httpRequest/abortMultipartUpload')
var listPartsRequest = require('./lib/httpRequest/listPartsRequest')
var completeMultipartUploadRequest = require('./lib/httpRequest/completeMultipartUploadRequest')
var listMultipartUploadRequest = require('./lib/httpRequest/listMultipartUploadRequest')

NOSClient = function () {
    this.host = null
    this.port = null
    this.secretKey = null
    this.accessId = null
}

NOSClient.prototype.setHost = function (host) {
    this.host = host;
}

NOSClient.prototype.setPort = function (port) {
    this.port = port;
}

NOSClient.prototype.setAccessId = function (accessId) {
    this.accessId = accessId
}

NOSClient.prototype.setSecretKey = function (secretKey) {
    this.secretKey = secretKey
}

NOSClient.prototype.abort_multipart_upload = function (map,func) {
    var request =  new abortUploadPartRequest()
    request.setHost(this.host)
    request.setPort(this.port)
    request.setAccessId(this.accessId)
    request.setSecretKey(this.secretKey)
    request.setBucketName(map['bucket'])
    request.setObjectName(map['key'])
    request.setUploadId(map['upload_id'])
    request.sendRequest(func)
}

NOSClient.prototype.complete_multipart_upload = function (map,func) {
    var request = new completeMultipartUploadRequest()
    request.setHost(this.host)
    request.setPort(this.port)
    request.setAccessId(this.accessId)
    request.setSecretKey(this.secretKey)
    request.setBucketName(map['bucket'])
    request.setObjectName(map['key'])
    request.setUploadId(map['upload_id'])
    request.setPartArray(map['info'])
    request.setObjectMD5(map['object_md5'])
    request.sendRequest(func)
}

NOSClient.prototype.delete_objects = function (map,func) {
    var request = new deleteMultipleObjectsRequest()
    request.setHost(this.host)
    request.setPort(this.port)
    request.setAccessId(this.accessId)
    request.setSecretKey(this.secretKey)
    request.setBucketName(map['bucket'])
    request.setObjectArray(map['keys'])
    request.setQuiet(map['quiet'])
    request.sendRequest(func)
}

NOSClient.prototype.delete_object = function (map,func) {
    var request = new deleteObjectRequest()
    request.setHost(this.host)
    request.setPort(this.port)
    request.setAccessId(this.accessId)
    request.setSecretKey(this.secretKey)
    request.setBucketName(map['bucket'])
    request.setObjectName(map['key'])
    request.sendRequest(func)
}

NOSClient.prototype.get_object = function (map,func) {
    var request = new getObjectRequest()
    request.setHost(this.host)
    request.setPort(this.port)
    request.setAccessId(this.accessId)
    request.setSecretKey(this.secretKey)
    request.setBucketName(map['bucket'])
    request.setObjectName(map['key'])
    request.setRange(map['range'])
    request.setIfModifiedSince(map['if_modified_since'])
    request.sendRequest(func)
}

NOSClient.prototype.head_object = function (map,func) {
    var request = new headObjectRequest()
    request.setHost(this.host)
    request.setPort(this.port)
    request.setAccessId(this.accessId)
    request.setSecretKey(this.secretKey)
    request.setBucketName(map['bucket'])
    request.setObjectName(map['key'])
    request.setIfModifiedSince(map['if_modified_since'])
    request.sendRequest(func)
}

NOSClient.prototype.create_multipart_upload = function (map,func) {
    var request = new initiateMultipartUploadRequest()
    request.setHost(this.host)
    request.setPort(this.port)
    request.setAccessId(this.accessId)
    request.setSecretKey(this.secretKey)
    request.setBucketName(map['bucket'])
    request.setObjectName(map['key'])
    request.setStorageClass(map['x-nos-storage-class'])
    if (map['meta_data'] != null){
        for (var key in map['meta_data']){
            request.setCustomMetaData(key,map['meta_data'][key])
        }
    }
    request.sendRequest(func)
}

NOSClient.prototype.list_multipart_upload = function (map,func) {
    var request = new listMultipartUploadRequest()
    request.setHost(this.host)
    request.setPort(this.port)
    request.setAccessId(this.accessId)
    request.setSecretKey(this.secretKey)
    request.setBucketName(map['bucket'])
    request.setMaxUploads(map['limit'])
    request.setKeyMarker(map['key_marker'])
    request.sendRequest(func)
}

NOSClient.prototype.list_objects = function (map,func) {
    var request = new listObjectsRequest()
    request.setHost(this.host)
    request.setPort(this.port)
    request.setAccessId(this.accessId)
    request.setSecretKey(this.secretKey)
    request.setBucketName(map['bucket'])
    request.setDelimiter(map['delimiter'])
    request.setMarker(map['marker'])
    request.setMaxKeys(map['limit'])
    request.setPrefix(map['prefix'])
    request.sendRequest(func)
}

NOSClient.prototype.list_parts = function (map,func) {
    var request = new listPartsRequest()
    request.setHost(this.host)
    request.setPort(this.port)
    request.setAccessId(this.accessId)
    request.setSecretKey(this.secretKey)
    request.setBucketName(map['bucket'])
    request.setObjectName(map['key'])
    request.setMaxParts(map['limit'])
    request.setPartNumberMarker(map['part_number_marker'])
    request.setUploadId(map['upload_id'])
    request.sendRequest(func)
}

NOSClient.prototype.copy_object = function (map,func) {
    var request = new putObjectCopyRequest()
    request.setHost(this.host)
    request.setPort(this.port)
    request.setAccessId(this.accessId)
    request.setSecretKey(this.secretKey)
    request.setSourceBucketName(map['src_bucket'])
    request.setSourceObjectName(map['src_key'])
    request.setDestBucketName(map['dest_bucket'])
    request.setDestObjectName(map['dest_key'])
    request.sendRequest(func)
}

NOSClient.prototype.move_object = function (map,func) {
    var request = new putObjectMoveRequest()
    request.setHost(this.host)
    request.setPort(this.port)
    request.setAccessId(this.accessId)
    request.setSecretKey(this.secretKey)
    request.setSourceBucketName(map['src_bucket'])
    request.setSourceObjectName(map['src_key'])
    request.setDestBucketName(map['dest_bucket'])
    request.setDestObjectName(map['dest_key'])
    request.sendRequest(func)
}

NOSClient.prototype.put_object = function (map,func) {
    var request = new putObjectRequest()
    request.setHost(this.host)
    request.setPort(this.port)
    request.setAccessId(this.accessId)
    request.setSecretKey(this.secretKey)
    request.setBucketName(map['bucket'])
    request.setObjectName(map['key'])
    request.setBody(map['body'])
    request.setStorageClass(map['storage_class'])
    if (map['meta_data'] != null){
        for (var key in map['meta_data']){
            request.setCustomMetaData(key,map['meta_data'][key])
        }
    }
    request.sendRequest(func)
}

NOSClient.prototype.upload_part = function (map,func) {
    var request = new uploadPartRequest()
    request.setHost(this.host)
    request.setPort(this.port)
    request.setAccessId(this.accessId)
    request.setSecretKey(this.secretKey)
    request.setBucketName(map['bucket'])
    request.setObjectName(map['key'])
    request.setBody(map['body'])
    request.setPartNumber(map['part_num'])
    request.setUploadId(map['upload_id'])
    request.sendRequest(func)
}

module.exports = NOSClient

